﻿using C1.Chart;
using C1.WPF.Chart;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ArrangeDataLabels
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        List<TextBlock> _labels = new List<TextBlock>();
        public MainWindow()
        {
            InitializeComponent();
        }

        private void FlexChart_LabelRendered(object sender, C1.WPF.Chart.RenderDataLabelEventArgs e)
        {
            var labels = GetLabelElementsWithText(e.Text);
            foreach (var label in labels)
            {
                if (!_labels.Contains(label))
                {
                    label.Loaded += Label_Loaded;
                    _labels.Add(label);
                }

            }
        }

        private void Label_Loaded(object sender, RoutedEventArgs e)
        {
            TextBlock label = sender as TextBlock;
            var axisRect = (flexChart.AxisX as IAxis).Rect;

            if (label.RenderTransform != null)
            {
                MatrixTransform mt = label.RenderTransform as MatrixTransform;

                var position = new Point(Canvas.GetLeft(label), Canvas.GetTop(label));
                var size = new Size(label.ActualWidth, label.ActualHeight);
                var isLabelInside = axisRect.Top <= (position.Y + size.Width) + 15; //Consider 15 is minimum position above the X-axis
                if (isLabelInside)
                {
                    label.RenderTransform = new MatrixTransform(
                        mt.Matrix.M11,
                        mt.Matrix.M12,
                        mt.Matrix.M21,
                        mt.Matrix.M22,
                        mt.Matrix.OffsetX,
                        mt.Matrix.OffsetY - (position.Y + size.Width - axisRect.Top) - 15); //Consider 15 is minimum position above the X-axis
                }
            }
        }

        private List<TextBlock> GetLabelElementsWithText(string text)
        {
            RenderCanvas canvas = flexChart.Template.FindName("canvas", flexChart) as RenderCanvas;
            List<TextBlock> dataLabels = new List<TextBlock>();

            foreach(var item in canvas.Children)
            {
                if(item is TextBlock)
                {
                    var tb = item as TextBlock;
                    if(tb.Text == text)
                    {
                        dataLabels.Add(tb);
                    }
                }
            }
            return dataLabels;
        }

        private void FlexChart_Rendered(object sender, RenderEventArgs e)
        {
            _labels.Clear();
        }

        private void BtnAdd_Click(object sender, RoutedEventArgs e)
        {
            flexChart.Series.Add(new Series
            {
                SeriesName = "Prev Median Value",
                Binding = "PrevMedianValue",
            });
            btnAdd.IsEnabled = false;
            btnRemove.IsEnabled = true;
        }

        private void BtnRemove_Click(object sender, RoutedEventArgs e)
        {
            flexChart.Series.RemoveAt(1);
            btnAdd.IsEnabled = true;
            btnRemove.IsEnabled = false;
        }
    }
    public class DataManager
    {
        List<DataItem> _data;
        public List<DataItem> Data
        {
            get
            {
                if(_data == null)
                {
                    InitData();
                }
                return _data;
            }
        }

        private void InitData()
        {
            _data = new List<DataItem>();
            _data.Add(new DataItem { X = 1, MedianValue = 315400, PrevMedianValue = 275000 });
            _data.Add(new DataItem { X = 2, MedianValue = 329300, PrevMedianValue = 276000});
            _data.Add(new DataItem { X = 3, MedianValue = 4105, PrevMedianValue = 4105});
            _data.Add(new DataItem { X = 4, MedianValue = 326100, PrevMedianValue = 277700});
            _data.Add(new DataItem { X = 5, MedianValue = 63600, PrevMedianValue = 62000});
            _data.Add(new DataItem { X = 6, MedianValue = 45100, PrevMedianValue = 44500});
            _data.Add(new DataItem { X = 7, MedianValue = 142950, PrevMedianValue = 125000});
            _data.Add(new DataItem { X = 8, MedianValue = 319750, PrevMedianValue = 235500});
            _data.Add(new DataItem { X = 9, MedianValue = 185162, PrevMedianValue = 150000});
            _data.Add(new DataItem { X = 10, MedianValue = 5000, PrevMedianValue = 5000});
            _data.Add(new DataItem { X = 11, MedianValue = 734, PrevMedianValue = 734});
        }
    }
    public class DataItem
    {
        public int X { get; set; }
        public int PrevMedianValue { get; set; }
        public int MedianValue { get; set; }
    }
}
